"use strict";
const hrtime_now_1 = require("hrtime-now");
const typescript_ast_util_1 = require("typescript-ast-util");
const typescript_plugin_util_1 = require("typescript-plugin-util");
const reorderParams_1 = require("./reorderParams");
const reorderParamsPlugin_1 = require("./reorderParamsPlugin");
const log = (msg) => info.project.projectService.logger.info(`${reorderParamsPlugin_1.PLUGIN_NAME} ${msg}`);
let uiTool;
function getCompletionsAtPosition(fileName, position, options) {
    const prior = info.languageService.getCompletionsAtPosition(fileName, position, options);
    if (prior) {
        prior.entries = prior.entries.concat(uiTool.getCompletionsAtPosition(fileName, position, options));
    }
    return prior;
}
let selectedAction;
function getApplicableRefactors(fileName, positionOrRange, userPreferences) {
    const t0 = hrtime_now_1.now();
    const result = uiTool.getApplicableRefactors(info, `${reorderParamsPlugin_1.PLUGIN_NAME}-refactor`, reorderParamsPlugin_1.REFACTOR_ACTION_NAME, fileName, positionOrRange, userPreferences);
    log(`getApplicableRefactors took ${hrtime_now_1.timeFrom(t0)}`);
    selectedAction = result.selectedAction;
    return result.refactors;
}
// TODO: This should be agnostic to any plugin such as proactive's
function getEditsForRefactor(fileName, formatOptions, positionOrRange, refactorName, actionName, userPreferences) {
    const t0 = hrtime_now_1.now();
    const refactors = info.languageService.getEditsForRefactor(fileName, formatOptions, positionOrRange, refactorName, actionName, userPreferences);
    if (actionName !== reorderParamsPlugin_1.REFACTOR_ACTION_NAME) {
        return refactors;
    }
    try {
        const simpleProject = typescript_plugin_util_1.createSimpleASTProject(info.project);
        const sourceFile = simpleProject.getSourceFileOrThrow(fileName);
        const funcDecl = reorderParamsPlugin_1.getFunctionSimple(sourceFile, typescript_ast_util_1.positionOrRangeToNumber(positionOrRange), selectedAction.args.name);
        log(`getEditsForRefactor ${funcDecl && funcDecl.getKindName()} [${selectedAction.args && selectedAction.args.reorder && selectedAction.args.reorder.join(', ')}] ${funcDecl && funcDecl.getText()}`);
        reorderParams_1.reorderParameters(funcDecl, selectedAction.args.reorder);
        sourceFile.saveSync();
    }
    catch (error) {
        log(`getEditsForRefactor error ${error + ' - ' + error.stack}`);
        return refactors;
    }
    log(`getEditsForRefactor ${selectedAction.name} took  ${hrtime_now_1.timeFrom(t0)}`);
}
let info;
const pluginDefinition = {
    getApplicableRefactors, getEditsForRefactor, getCompletionsAtPosition
};
module.exports = typescript_plugin_util_1.getPluginCreate(pluginDefinition, (modules, anInfo) => {
    uiTool = reorderParamsPlugin_1.getTextUITool({ log, program: info.languageService.getProgram() });
    info = anInfo;
    log(`created`);
});
//# sourceMappingURL=index.js.map