"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const ts = __importStar(require("typescript"));
const typescript_ast_util_1 = require("typescript-ast-util");
const typescript_plugins_text_based_user_interaction_1 = require("typescript-plugins-text-based-user-interaction");
const ts_simple_ast_1 = require("ts-simple-ast");
exports.PLUGIN_NAME = 'typescript-plugin-function-signature-refactors';
exports.REFACTOR_ACTION_NAME = `${exports.PLUGIN_NAME}-refactor-action`;
let interactionTool;
function getTextUITool(config) {
    const reorderParamsTextUIConfig = {
        prefix: '&%&%',
        printHelp: true,
        log: (msg) => {
            config.log(`${exports.PLUGIN_NAME} interaction-tool ${msg}`);
        },
        actions: [
            {
                name: 'reorderParams',
                args: ['name', 'reorder'],
                print: action => `Reorder parameters of "${action.args.name}"`,
                snippet: (fileName, position) => {
                    let func = getFunction(fileName, position, config.program);
                    if (!func || func.parameters && func.parameters.length <= 1) {
                        return;
                    }
                    const reorder = [];
                    for (let i = 0; i < func.parameters.length; i++) {
                        reorder.push(func.parameters.length - i - 1);
                    }
                    return `reorderParams("${func.name.getText()}", [${reorder.join(', ')}])
  
  /* Help: The second argument is the new order of parameters. Number N in index M means move the M-th 
      argument to index N. Examples: 
   * [1] means switch between first and second
   * [3, 2] means move the first parameter to fourth position and move the second parameter to the third. 
   *   (the rest of the parameters, (third and fourth) will move left to accommodate this requirements) */`;
                },
                // TODO: we could give a more intuitive text-based API by letting the user provide the new signature. Then we create a new function with that signature in order to parse it correctly. 
                nameExtra: (fileName, position) => {
                    const func = getFunction(fileName, position, config.program);
                    return func ? `of ${func.name.getText()}` : '';
                }
            }
        ]
    };
    if (!interactionTool) {
        interactionTool = typescript_plugins_text_based_user_interaction_1.create(reorderParamsTextUIConfig);
    }
    return interactionTool;
}
exports.getTextUITool = getTextUITool;
function getFunction(fileName, position, program) {
    // TODO: cache
    const sourceFile = program.getSourceFile(fileName);
    const node = typescript_ast_util_1.findChildContainingRangeLight(sourceFile, typescript_ast_util_1.positionOrRangeToRange(position));
    if (!node) {
        return;
    }
    const expr = typescript_ast_util_1.findAscendant(node, ts.isExpressionStatement, true);
    if (!expr) {
        return;
    }
    return [typescript_ast_util_1.getNextSibling(expr), typescript_ast_util_1.getPreviousSibling(expr)].find(ts.isFunctionLike);
}
/** same as getFunction but in ts-simple-ast project */
function getFunctionSimple(file, position, name) {
    let expr = file.getDescendantAtPos(position);
    const e = [expr].concat(expr.getAncestors()).find(e => ts_simple_ast_1.TypeGuards.isSignaturedDeclaration(e) && ts_simple_ast_1.TypeGuards.isNamedNode(e) && e.getName() === name);
    if (ts_simple_ast_1.TypeGuards.isSignaturedDeclaration(e) && ts_simple_ast_1.TypeGuards.isNamedNode(e)) {
        return e;
    }
}
exports.getFunctionSimple = getFunctionSimple;
//# sourceMappingURL=reorderParamsPlugin.js.map